import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import controlP5.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class MIGNON_DotMatrixEditor1 extends PApplet {

// Mignon Dot Matrix Editor 1.0
// For Mignon Game Kit and DISPL4Y
// CC BY-SA 3.0 DE
// Olaf Val

// Libraries:
// 1. controlP5

int ro = 5;
int co = 7;
int ra = 15;   // radius
int di = 20;   // distance
int posY = 30; // dot matrix position 
int tx;        // dot x
int ty;        // dot y

int[] dl = new int[256];
int[] dlex = new int[256];

int gap;
int lastD = -1; // last dot clicked on
int dataCol = 200; // Data backgroundColor
int timer = millis();
int cKey = 0; // command key pressend
int helpOn = 0; // Help on / off

String str = ""; // dot-list as string

PFont font;
PFont font14;
PImage logo;

// GUI

ControlP5 cp5;
DropdownList deviceList;
int dimmVal = 15;

public void setup() {
  size(265, 320);
  background(120);
  logo = loadImage("Logo_Mignon_Farbe_100.png");

  // set all dots to 0
  clearAll();

  font = loadFont("AUdimat-Regular-10.vlw");
  font14 = loadFont("AUdimat-Regular-14.vlw");

  // setUp GUI
  cp5 = new ControlP5(this);

  // myDeviceList
  deviceList = cp5.addDropdownList("myDeviceList")
    .setPosition(140 + ra, 70);
  customize(deviceList);

  // Dimm Slider
  cp5.addSlider("dimm")
    .setRange(0, 15)
      .setValue(dimmVal)
        .setPosition(140 + ra, 80)
          .setSize(76, 10)
            ;

  // Up = Dimmer up
  cp5.mapKeyFor(new ControlKey() {
    public void keyEvent() {
      dimmVal = PApplet.parseInt( cp5.getController("dimm").getValue() );
      dimmVal++;
      cp5.getController("dimm").setValue(dimmVal);
    }
  }
  , UP);

  // Down = Dimmer down
  cp5.mapKeyFor(new ControlKey() {
    public void keyEvent() {
      dimmVal = PApplet.parseInt( cp5.getController("dimm").getValue() );
      dimmVal--;
      cp5.getController("dimm").setValue(dimmVal);
    }
  }
  , DOWN);

  // Clear Button
  cp5.addButton("clear")
    .setPosition(140 + ra, 105)
      .setSize(100, 15)
        .setLabel("clear all");

  // key = DELETE or BACKSPACE
  cp5.mapKeyFor(new ControlKey() {
    public void keyEvent() {
      clearAll();
    }
  }
  , DELETE);

  cp5.mapKeyFor(new ControlKey() {
    public void keyEvent() {
      clearAll();
    }
  }
  , BACKSPACE);

  // Copy to Chlipboard Button
  cp5.addButton("copy")
    .setPosition(140 + ra, 130)
      .setSize(100, 15)
        .setLabel("Copy To Clipboard");

  // Import from Chlipboard Button
  cp5.addButton("imp")
    .setPosition(140 + ra, 155)
      .setSize(100, 15)
        .setLabel("Import from Clipboard");

  // Help Button
  cp5.addButton("help")
    .setPosition(140 + ra, 180)
      .setSize(100, 15)
        .setLabel("help");
}

public void draw() {
  background(120);
  image(logo, 155, 12);
  textFont(font14);
  text("Dot Matrix Editor", 154, 42);

  // Dot Display - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  int c = 0;
  for (int f2 = 0; f2 < ro; f2++) {
    for (int f1 = 0;f1 < co; f1++) {
      tx = di + (di*f1);
      ty = di + (di*f2);

      fill(230);
      if (
      (mouseX > tx - (ra/2) )&&
        (mouseX < tx + (ra/2) )&&
        (mouseY > ty - (ra/2) )&&
        (mouseY < ty + (ra/2) )
        ) 
      {
        fill(255);
        gap = 1;
        if ( (mousePressed) && (dlex[c] != 1) ) {
          dlex[c] = 1;
          if (dl[c] == 15) {
            dl[c] = 0;
            lastD = -1;
          }
          else {
            dl[c] = 15;
            lastD = c;
          }
        }
      }
      else {
        gap = 0;
      }

      if ( (mousePressed)||(gap == 0) ) {
      }
      else { 
        for (int f3=0; f3<256; f3++) {       
          dlex[f3] = 0;
        }
      }

      fill(map(dl[c], 0, 15, 0, 255), 0, 0);

      if (c == lastD) ellipse(tx, ty, ra+2, ra+2);
      ellipse(tx, ty, ra, ra);

      c++;
    }
  }

  // Text Box - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  if (millis() > timer + 10) {
    timer = millis();
    if (dataCol < 200) dataCol++;
  }
  fill(dataCol);
  noStroke();
  rect(12, ty + ra, 140, 150);
  textFont(font); 
  fill(10);
  noStroke();
  //text("Data:", 15, 125);

  c = 0;
  for (int f2=0; f2<ro; f2++) {
    for (int f1=0; f1<co; f1++) {

      text(dl[c], 15 + (f1*20), (ty + ra + 20) + (f2 * 20));
      c++;
    }
  }

  if (helpOn == 1) helpBox();
}

public void customize(DropdownList ddl) {
  // a convenience function to customize a DropdownList
  ddl.setBackgroundColor(color(190));
  ddl.setItemHeight(20);
  ddl.setBarHeight(15);
  ddl.captionLabel().set("Device");
  ddl.captionLabel().style().marginTop = 3;
  ddl.captionLabel().style().marginLeft = 3;
  ddl.valueLabel().style().marginTop = 3;

  ddl.addItem("Game Kit", 1);
  ddl.addItem("DISPL4Y", 2);
  //ddl.addItem("No Device", 3);

  //ddl.scroll(0);
  ddl.setColorBackground(color(60));
  ddl.setColorActive(color(255, 128));
}

public void controlEvent(ControlEvent theEvent) {

  if (theEvent.isGroup()) {
    // check if the Event was triggered from a ControlGroup
    println("event from group : "+ theEvent.getGroup().getValue() +" from "+theEvent.getGroup());
    if (theEvent.getGroup().getValue() == 1) { 
      println("Show Game Kit");
      ro = 5;
      co = 7;
    }
    if (theEvent.getGroup().getValue() == 2) {
      println("Show DISPL4Y");
      ro = 7;
      co = 5;
    }
  } 
  else if (theEvent.isController()) {
    println("event from controller : "+theEvent.getController().getValue()+" from "+theEvent.getController());
  }
}

public void dimm(int dimmIt) {
  if (lastD >= 0) dl[lastD] = dimmIt;
}

public void clear() {
  clearAll();
}

public void copy() {
  copyClipboard();
}

public void copyClipboard() {
  dataCol = 150;
  // dot-list to string
  str = "";

  int c = 0;
  for (int f2=0; f2<ro; f2++) {
    for (int f1=0; f1<co; f1++) {
      str = str + str(dl[c]);
      if (c < (ro * co) -1 ) str = str + ", ";
      c++;
    }
    if (c < (ro * co) -1 ) str = str + "\n";
  }

  println("");
  println(str);

  // Copy to clipboard
  GClip.copy(str);
}

public void imp() {
  impClipboard();
}

public void impClipboard() {
  dataCol = 150;

  // Import from clipboard
  str = "";
  str = GClip.paste();


  // string to dot-list
  //readl = int(split(str[0],','));
  //String[] inString = split(
  int[] readl = PApplet.parseInt(split(str, ", "));
  if (readl.length == co * ro) {
    for (int f3 = 0; f3 < (co * ro); f3++) {
      dl[f3] = readl[f3];
      println(readl[f3]);
    }
  }
}

public void clearAll() {
  // set all dots to 0
  for (int f=0; f < 256; f++) {
    dl[f] = 0;
  }
}

public void help() {
  helpOn = 1;
}

public void helpBox() {
  fill(160);
  rect(5, 5, 140, 255);
  if (mousePressed) helpOn = 0;
  textFont(font); 
  fill(10);
  text("X", 137, 15);
  text(
  "Edit your bitmaps by clicking" + "\n" +
  "on the dots." + "\n" + "\n" +
  "The last dot can be dimmed" + "\n" +
  "by the dimm slider." + "\n" + "\n" +
  "Copy your bitmap code and" + "\n" +
  "paste it into the Arduino" + "\n" +
  "gamekit load image example." + "\n" + "\n" +
  "Keys:" + "\n" + 
  "Up = Dimm Up" + "\n" +
  "Down = Dimm Down" + "\n" +
  "Backspace = Clear All" + "\n" +
  "Command + C = Copy" + "\n" +
  "Command + V = Paste" + "\n" +
  "Command + H = Help"
  , 10, 20);
}

public void keyPressed() {
  if (key == CODED) {
    if ( (keyCode == 157) || (keyCode == 17) || (keyCode == ALT) ) {
      cKey = 1;
    }
  }
  if (cKey == 1) {
    if (key == 'd') clearAll();
    if (key == 'c') copyClipboard();
    if (key == 'v') impClipboard();
  }
}

public void keyReleased() {
  if (key == CODED) {
    //println(keyCode);
    if ( (keyCode == 157) || (keyCode == 17) || (keyCode == ALT) ) {
      cKey = 0;
      println("0");
    }
  }
}

  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "MIGNON_DotMatrixEditor1" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
